# ###################################################
# Data Management in the "tidyverse" with R Packages:
# tidyr and dplyr
# Boriana Pratt and Dawn Koffman - May 2018
# ###################################################

# slide 19
# ########
# install.packages("tidyr")
# install.packages("dplyr")
# install.packages("ggplot2")
# install.packages("gapminder")

library("tidyr")
library("dplyr")
library("ggplot2")
library("gapminder")

usle <- read.csv(file="uslifeexp.csv", head=TRUE, sep=",")
usle

# slide 20
# ########
gather(usle, key="sex", value="lifeexp", le_male, le_female)

# "pipe operator" ... think of "then"

select(usle, year, le_male, le_female) %>% 
  gather(key="sex", value="lifeexp", le_male, le_female) %>% arrange(year)

# slide 21
# ########

select(usle, year, le_male, le_female) %>% 
  rename(male = le_male, female = le_female) %>% 
  gather(key="sex", value="lifeexp", male, female) %>%
  arrange(year)

# slide 22
# ########
# EXERCISE: 
# Display tidy data that shows 
# life expectancy by race (black, white)


# EXTRA CREDIT EXERCISE:
# Using your tidy data along with additional "piping" ... 
# display a ggplot2 graph that shows life expectancy by race (black, white) 


# slide 24
# ########
# show life expectancy for black, white, male, female on same graph
mfle <- select(usle, year, le_male, le_female) %>% 
  rename(male = le_male, female = le_female) %>% 
  gather(key="sex", value="lifeexp", male, female) %>%
  arrange(year)

select(usle, year, le_b, le_w) %>% 
  rename(black = le_b, white = le_w) %>% 
  gather(key="race", value="lifeexp", black, white) %>%
  arrange(year) %>% 
  ggplot(aes(year, lifeexp, color= race)) + geom_point() +
  geom_point(data = mfle, aes(color = sex )) +
  scale_color_discrete(name="Group")

# slide 25
# ########
# use mutate to create gap between male and female, 
# and between black and white, 

mutate(usle, race_le_gap = le_w - le_b, sex_le_gap = le_female - le_male) %>%
  select(year, race_le_gap, sex_le_gap)

# is above data tidy?

mutate(usle, race_le_gap = le_w - le_b, sex_le_gap = le_female - le_male) %>%
  select(year, race_le_gap, sex_le_gap) %>% 
  ggplot(aes(year, race_le_gap)) + 
  geom_point(color="red") + 
  geom_point(aes(y=sex_le_gap), color="blue")  + 
  ylab("life expectancy gap")

# slide 26
# ########
# how to tidy this data?
mutate(usle, race = le_w - le_b, sex = le_female - le_male) %>%
  select(year, race, sex) %>% 
  gather(key="le_gap_type", value="le_gap_years", race, sex)

# and then use tidy data to draw graph?
mutate(usle, race = le_w - le_b, sex = le_female - le_male) %>%
  select(year, race, sex) %>% 
  gather(key="le_gap_type", value="le_gap_years", race, sex) %>%
  ggplot(aes(year, le_gap_years, color = le_gap_type)) + 
  geom_point()+ 
  ylab("life expectancy gap")

# slide 27
# ########
# Display tidy data where unit of observation is year-race-sex
select(usle, year, le_wmale, le_wfemale, le_bmale, le_bfemale) %>%
  rename(white_male = le_wmale, white_female = le_wfemale, black_male = le_bmale, 
         black_female = le_bfemale) %>%
  gather(key="race_sex", value="lifeexp", white_male, white_female, black_male, black_female) %>%
  arrange(year, race_sex)

select(usle, year, le_wmale, le_wfemale, le_bmale, le_bfemale) %>%
  rename(white_male = le_wmale, white_female = le_wfemale, black_male = le_bmale, 
         black_female = le_bfemale) %>%
  gather(key="race_sex", value="lifeexp", white_male, white_female, black_male, black_female) %>%
  arrange(year, race_sex) %>%
  ggplot(aes(year, lifeexp, color = race_sex)) + 
  geom_point() 

# slide 28
# ########
# Reminder - tidy data: each variable is saved in ITS OWN column
select(usle, year, le_wmale, le_wfemale, le_bmale, le_bfemale) %>%
  rename(white_male = le_wmale, white_female = le_wfemale, black_male = le_bmale, 
         black_female = le_bfemale) %>%
  gather(key="racesex", value="lifeexp", white_male, white_female, black_male, black_female) %>%
  arrange(year, racesex) %>%
  separate(racesex, c("race", "sex"), sep = "_") 

# slide 29
# ########
# tidyr function that goes in reverse direction: unite 
tidy_le <- select(usle, year, le_wmale, le_wfemale, le_bmale, le_bfemale) %>%
  rename(white_male = le_wmale, white_female = le_wfemale, black_male = le_bmale, black_female = le_bfemale) %>%
  gather(key="racesex", value="lifeexp", white_male, white_female, black_male, black_female) %>%
  arrange(year, racesex) %>%
  separate(racesex, c("race", "sex"), sep = "_") 

tidy_le
unite(tidy_le, "racesex", c(race, sex), sep = "_")

# slide 30
# ########
# tidyr function that goes in reverse direction: spread (rows -> columns; "long" to "wide")
unite(tidy_le, "racesex", c(race, sex), sep = "_") %>% 
  spread(key="racesex", value="lifeexp")
  
unite(tidy_le, "race_sex", c(race, sex), sep = "_") %>% 
  spread(key="race_sex", value="lifeexp") %>%
  rename(le_bfemale = black_female, le_bmale = black_male, le_wfemale = white_female, 
         le_wmale = white_male)

# slide 31
# ########
# EXERCISE:
# list data in tidy format with group as one variable 
# (taking values: 
# all, male, female, black, white, white_male, white_female, black_male, black_female), 
# and life_expectancy as the other variable.


# slide 33
# ########
# preview a bit more dplyr: filter rows
rename(usle, all=le, male=le_male, female=le_female, black=le_b, white=le_w, 
       white_male=le_wmale, white_female=le_wfemale, black_male=le_bmale, black_female=le_bfemale) %>%
  gather(key = "group", value = "life_expectancy", 2:10) %>%
  arrange(year, group) %>%
  filter(group == "black" | group == "white")

# slide 34
# ########
rename(usle, all=le, male=le_male, female=le_female, black=le_b, white=le_w, 
                     white_male=le_wmale, white_female=le_wfemale, 
                     black_male=le_bmale, black_female=le_bfemale) %>%
  gather(key = "group", value = "life_expectancy", 2:10) %>%
  arrange(year, group) %>%
  filter(group == "male" | group == "female") %>%
  ggplot(aes(year, life_expectancy, color = group)) + geom_point()

# slide 35
# ########
rename(usle, all=le, male=le_male, female=le_female, black=le_b, white=le_w, 
             white_male=le_wmale, white_female=le_wfemale, 
             black_male=le_bmale, black_female=le_bfemale) %>%
  gather(key = "group", value = "life_expectancy", 2:10) %>%
  arrange(year, group) %>%
  filter(group %in% c("white_male", "white_female", "black_male", "black_female")) %>%
  ggplot(aes(year, life_expectancy, color = group)) + geom_point()

# slide 38
# ########
# check structure of gapminder data
str(gapminder)

# tbl_df: improved data.frame for which dplyr provides nice methods for high-level inspection
# these methods do something sensible for datasets with many observations and/or variables

gdf <- as.data.frame(gapminder)
str(gdf)

gtdf <- tbl_df(gdf)
str(gtdf)

# high-level inspection of tbl_df
glimpse(gapminder)
View(gapminder) # note capital V

# slide 40
# ########
# subset observations

filter(gapminder, country == "United States")

filter(gapminder, lifeExp < 30)

filter(gapminder, pop < 1000000 )

filter(gapminder, pop < 1000000, year == 2007)

filter(gapminder, pop < 1000000 & year == 2007)

filter(gapminder, country == "United States" | country == "Canada", year > 2000)

filter(gapminder, country %in% c("United States", "Canada"), year > 2000)

distinct(gapminder, country)
View(distinct(gapminder, country))
distinct(gapminder, country) %>% View()

distinct(as.data.frame(gapminder), country)

# slide 41
# ########
# subset columns

select(gapminder, country, continent)

country_continent <- select(gapminder, country, continent) %>% distinct()
country_continent

select(gapminder, -continent) # "-" means not ... gives TIDIER data set
tgap <- select(gapminder, -continent)

# But how to combine tgap and country_continent when want 
# to summarize values by continent??? 
# Will later use a "join" function to combine

select(gapminder, year, country, continent, lifeExp) # select and re-order columns

select(gapminder, starts_with("co")) # select column headers that start with "co"

select(gapminder, country:lifeExp) # range

# slide 42
# ########
# EXERCISE: 
# list all countries showing only life expectancy for 2007 


# EXTRA CREDIT EXERCISE: 
# list all countries showing only life expectancy for 2007
# with life expectancy variable named le (rather than lifeExp)


# slide 44
# ########
# order rows
arrange(gapminder, year)

rename(gapminder, le = lifeExp) %>% filter(year == 2007) %>% 
  select(country, year, le) %>% arrange(le)

rename(gapminder, le = lifeExp) %>% filter(year == 2007) %>% 
  select(country, year, le) %>% arrange(desc(le)) # order by descending le

# to list all rows, can use gapminder as a data frame
rename(as.data.frame(gapminder), le = lifeExp) %>% 
  filter(year == 2007) %>%
  select(country, year, le) %>% arrange(desc(le))

# list 5 countries with highest life expectancy in 2007 
# show country, year, and le
rename(gapminder, le = lifeExp) %>%
  filter(year == 2007) %>% 
  top_n(5, le) %>%  # filter rows again
  select(country, year, le) %>%
  arrange(desc(le)) # sort top 5

# slide 45
# ########
# EXERCISE: 
# list 10 lowest life expectancies, with lowest at the top
# HINT: use a second filter command


# slide 47
# ########
# construct new columns

mutate(gapminder, popMil = round(pop / 1000000, 1), le = round(lifeExp, 1))

transmute(gapminder, country = country, y = year, 
                     popMil = round(pop / 1000000, 1), le = round(lifeExp, 1)) %>%
  arrange(y, country)

# slide 48
# ########
# use vector functions to create new columns
# vector functions take a vector of n values and return n values
# types of vector functions:
#   - ranking and ordering functions 
#   - cumulative aggregates
#   - access to previous and next values 

filter(gapminder, year == 2007) %>% 
  mutate(le_rank = dense_rank(lifeExp)) %>%
  select(country, continent, year, lifeExp, le_rank) %>%
  arrange(le_rank)


# how to assign lowest rank to highest life expectancy?
filter(gapminder, year == 2007) %>% 
  mutate(le_rank = dense_rank(-lifeExp)) %>%
  select(country, continent, year, lifeExp, le_rank) %>%
  arrange(le_rank)

# slide 49
# ########
# vector functions: cumulative sum
filter(gapminder, year == 1952) %>% 
  arrange(continent, country) %>%
  mutate(popMil = round(pop / 1000000, 1)) %>%
  mutate(cumpopMil = cumsum(popMil)) %>% View()


filter(gapminder, year == 2007) %>% 
  arrange(continent, country) %>%
  mutate(popMil = round(pop / 1000000, 1)) %>%
  mutate(cumpopMil = cumsum(popMil)) %>% View()

# slide 50
# ########
# group data: construct new column values by group

filter(gapminder, year == 2007) %>% 
  mutate(popMil = round(pop / 1000000, 1)) %>%
  arrange(continent, popMil) %>%
  group_by(continent) %>%
  mutate(cumpopMil = cumsum(popMil)) %>% View()

select(gapminder, country, year, pop) %>%
  group_by(country) %>%
  mutate(pop_lag = lag(pop), pop_chg = pop - pop_lag, 
         pop_pctchg = round(pop_chg/pop_lag * 100, 1)) %>% View()

# slide 51
# ########
# EXERCISE:
# list only rows that experienced a population decline during the previous 5-years
# show country, year, pop, pop_chg, pop_pctchg


# slide 53
# ########
# summarise data
# use summarise() with a summary function to change the unit of observation
# summary functions take a vector of values and return a single value
# very often used with group_by()

filter(gapminder, year == 2007) %>%
  summarise(year = mean(year), ncountries = n(), 
  avg_country_le = mean(lifeExp), sd_country_le = sd(lifeExp))
 
filter(gapminder, year == 2007) %>% 
  group_by(continent) %>%
  summarise(avg_country_le = mean(lifeExp))

filter(gapminder, year == 2007) %>% 
  group_by(continent) %>%
  summarise(year = mean(year), ncountries = n(), 
  avg_country_le = mean(lifeExp), sd_country_le = sd(lifeExp))

filter(gapminder, year == 1952) %>% 
  group_by(continent) %>%
  summarise(year = mean(year), ncountries = n(), 
  avg_country_le = mean(lifeExp), sd_country_le = sd(lifeExp))

# slide 54
# ########
# EXERCISE 1: 
# show data by continent and years 1952 AND 2007
# list number of countries, avg_country_le, and sd_country_le


# EXERCISE 2: 
# By continent and year, show ncountries, avg_country_le, sd_country_le for all 12 years of data


# EXTRA CREDIT EXERCISE: 
# Make a simple graph that shows avg_country_le over time, 
# for each continent


# slide 56
# ########
# how many continents each country 
# has belonged to over time
group_by(gapminder, country) %>% 
  summarise(n_continents = n_distinct(continent)) 

# each summarise "peels off" one level of group_by()
group_by(gapminder, country) %>% 
  summarise(n_continents = n_distinct(continent)) %>% 
  summarise(avg_n_continents = mean(n_continents))

# slide 57
# ########
# further illustration of summarise() successively "peeling off" group_by()
group_by(gapminder, continent, country) %>%
  summarise(avg_le_cc = mean(lifeExp))

group_by(gapminder, continent, country) %>%
  summarise(avg_le_cc = mean(lifeExp)) %>%
  summarise(avg_le_c =  mean(avg_le_cc))

group_by(gapminder, continent, country)  %>%
  summarise(avg_le_cc = mean(lifeExp))   %>%
  summarise(avg_le_c  = mean(avg_le_cc)) %>%
  summarise(avg_le    = mean(avg_le_c))

# slide 58
# ########
# more summary functions

group_by(gapminder, country) %>%
  summarise(year = first(year), le = first(lifeExp))

group_by(gapminder, country) %>%
  summarise(year = last(year), le = last(lifeExp))

group_by(gapminder, country) %>%
  summarise(year = nth(year, 3), le = nth(lifeExp, 3)) 

# slide 59
# ########
# summarise_at():  
# Apply one or more functions to one or more columns. 
# Grouping variables are always excluded from modification. 

group_by(gapminder, continent, year) %>% 
summarise_at(c("lifeExp", "pop"), funs(min, median, max)) %>% View()

# ... and to re-order columns ... can use select:
group_by(gapminder, continent, year) %>%
  summarise_at(c("lifeExp", "pop"), funs(min, median, max)) %>%
  select(continent, year, lifeExp_min, lifeExp_median, lifeExp_max, pop_min, pop_median, pop_max) %>% View()

# slide 60
# ########
# graph min, median, max country life expectancy, by continent
group_by(gapminder, continent, year) %>%
  summarise_at(c("lifeExp", "pop"), funs(min, median, max)) %>%
  ggplot(aes(x=year, y = lifeExp_median)) + geom_line() +
  geom_line(aes(y = lifeExp_min), linetype = "dashed") +
  geom_line(aes(y = lifeExp_max), linetype = "dashed") + facet_grid(continent ~ .) +
  labs(y="Life Expectancy for Countries: min, median, max", x="")

# slide 61
# ########
# count() function wraps up the 
# common combination of group_by() and summarise():

# How many rows for each value of continent?
count(gapminder, continent)

# How many rows for each value of continent and year
count(gapminder, continent, year) %>% View()

filter(gapminder, year == 2002 | year == 2007) %>%
  count(continent)

# slide 63
# ########
# combining data sets

# saw above that every country has been associated
# with just one continent during time period
# so ...
# continent belongs in a table where unit of observation is country
# other variables belong in a table where unit of observation is country-year:

country_continent <- select(gapminder, country, continent) %>% distinct()
country_continent

tgap <- select(gapminder, -continent)
tgap

# BUT descriptive exploration has required 
# continent be included in data set
# HOW TO COMBINE ("join" or "merge") tgap and country_continent?

# join matching rows from second data set to first
left_join(tgap, country_continent, by = "country") %>% View()

# slide 64
# ########
country_continent_inc <- slice(country_continent, 6:142) # cut out rows 1-5
View(country_continent_inc)

tgap_inc <- slice(tgap, 49:144) # cut out rows 1-48 and rows 145-1704
View(tgap_inc)

# join matching rows from 2nd data set to first
left_join(tgap, country_continent_inc, by = "country") %>% View() 

# join matching rows from first data set to 2nd
right_join(tgap, country_continent, by = "country") %>% View()

right_join(tgap, country_continent_inc, by = "country") %>% View() # country_continent_inc is the driver!

# slide 65
# ########
# join and retain only rows in both data sets
inner_join(tgap_inc, country_continent_inc, by = "country") %>% View()

# join and retain all values, all rows
full_join(tgap_inc, country_continent_inc, by = "country") %>% View()

# retain all rows in first data set that have a match in second data set
# (but don't add columns)
semi_join(tgap_inc, country_continent_inc, by = "country") %>% View()

# retain all rows in first data set that do not have a match in second data set 
# (but don't add columns)
anti_join(tgap_inc, country_continent_inc, by = "country") %>% View()

# slide 66
# ########
# TO APPEND ROWS use bind_rows() ... more efficient than rbind()

tgap1992 <- filter(tgap, year == 1992) %>% select(-year)
tgap1997 <- filter(tgap, year == 1997) %>% select(-year)
tgap2002 <- filter(tgap, year == 2002) %>% select(-year)
tgap2007 <- filter(tgap, year == 2007) %>% select(-year)

tgap1992
tgap2007

bind_rows(tgap1992, tgap1997, tgap2002, tgap2007) %>% View() # ... OOPS .. not quite right!
bind_rows(list(tgap1992, tgap1997, tgap2002, tgap2007), .id="id") %>% View()  # ... a bit better!
bind_rows("1992" = tgap1992, "1997" = tgap1997, "2002" = tgap2002, "2007" = tgap2007, .id="year") %>% View()

